==> racer-0.1.1.tgz: package/lib/Store.js <==
var Field, MemoryAdapter, Model, Promise, PubSub, Serializer, Stm, Store, allOtPaths, pathParser, redis, redisInfo, specHelper, transaction;
redis = require('redis');
MemoryAdapter = require('./adapters/Memory');
Model = require('./Model.server');
Stm = require('./Stm');
PubSub = require('./PubSub');
transaction = require('./transaction');
pathParser = require('./pathParser');
Serializer = require('./Serializer');
specHelper = require('./specHelper');
redisInfo = require('./redisInfo');
Promise = require('./Promise');
Field = require('./mixin.ot/Field.server');
Store = module.exports = function(options) {
  var Adapter, adapter, addSubDatum, clientSockets, commit, db, forTxnSince, hasInvalidVer, host, ignoreSubscribe, localModels, nextClientId, nextTxnNum, onOtMsg, onTxnMsg, otFields, port, pubSub, redisClient, redisOptions, redisStarts, self, startIdPromise, stm, subClient, subscribeToStarts, txnApplier, txnSubClient, _ref;
  if (options == null) {
    options = {};
  }
  self = this;
  Adapter = options.Adapter || MemoryAdapter;
  this._adapter = adapter = new Adapter;
  _ref = redisOptions = options.redis || {}, port = _ref.port, host = _ref.host, db = _ref.db;
  this._redisClient = redisClient = redis.createClient(port, host, redisOptions);
  this._subClient = subClient = redis.createClient(port, host, redisOptions);
  this._txnSubClient = txnSubClient = redis.createClient(port, host, redisOptions);
  this.otFields = otFields = {};
  redisStarts = null;
  startIdPromise = new Promise;
  ignoreSubscribe = false;
  (subscribeToStarts = function(selected) {
    if (ignoreSubscribe) {
      return ignoreSubscribe = false;
    }
    if (db !== void 0 && !selected) {
      return redisClient.select(db, function(err) {
        if (err) {
          throw err;
        }
        return subscribeToStarts(true);
      });
    }
    return redisInfo.subscribeToStarts(subClient, redisClient, function(starts) {
      redisStarts = starts;
      if (startIdPromise.value) {
        startIdPromise.clearValue();
      }
      return startIdPromise.fulfill(starts[0][0]);
    });
  })();
  ignoreSubscribe = true;
  redisClient.on('connect', subscribeToStarts);
  redisClient.on('end', function() {
    redisStarts = null;
    return startIdPromise.clearValue();
  });
  this._localModels = localModels = {};
  onTxnMsg = function(clientId, txn) {
    var base, model, socket;
    if (clientId === transaction.clientId(txn)) {
      return;
    }
    if (model = localModels[clientId]) {
      return model._onTxn(txn);
    }
    if (socket = clientSockets[clientId]) {
      base = transaction.base(txn);
      if (base > socket.__base) {
        socket.__base = base;
        return nextTxnNum(clientId, function(num) {
          return socket.emit('txn', txn, num);
        });
      }
    }
  };
  onOtMsg = function(clientId, ot) {
    var socket;
    if (socket = clientSockets[clientId]) {
      if (socket.id === ot.meta.src) {
        return;
      }
      return socket.emit('otOp', ot);
    }
  };
  this._pubSub = pubSub = new PubSub({
    redis: redisOptions,
    pubClient: redisClient,
    subClient: txnSubClient,
    onMessage: function(clientId, _arg) {
      var ot, txn;
      txn = _arg.txn, ot = _arg.ot;
      if (txn) {
        return onTxnMsg(clientId, txn);
      }
      return onOtMsg(clientId, ot);
    }
  });
  this._nextTxnNum = nextTxnNum = function(clientId, callback) {
    return redisClient.incr('txnClock.' + clientId, function(err, value) {
      if (err) {
        throw err;
      }
      return callback(value);
    });
  };
  hasInvalidVer = function(socket, ver, clientStartId) {
    if (!startIdPromise.value) {
      socket.disconnect();
      return true;
    }
    if (!(clientStartId && clientStartId === startIdPromise.value)) {
      socket.emit('fatalErr');
      return true;
    }
    return false;
  };
  clientSockets = {};
  this._setSockets = function(sockets) {
    return sockets.on('connection', function(socket) {
      return socket.on('sub', function(clientId, paths, ver, clientStartId) {
        var txnsSince;
        if (hasInvalidVer(socket, ver, clientStartId)) {
          return;
        }
        socket.on('disconnect', function() {
          pubSub.unsubscribe(clientId);
          delete clientSockets[clientId];
          return redisClient.del('txnClock.' + clientId, function(err, value) {
            if (err) {
              throw err;
            }
          });
        });
        socket.on('subAdd', function(clientId, paths, callback) {
          pubSub.subscribe(clientId, paths);
          return self._fetchSubData(paths, function(err, data) {
            return callback(data);
          });
        });
        socket.on('subRemove', function(clientId, paths) {
          throw 'Unimplemented: subRemove';
        });
        socket.on('otSnapshot', function(setNull, fn) {
          var TODO, field;
          if (field = self.otFields[path]) {
            return TODO = 'TODO';
          }
        });
        socket.on('otOp', function(msg, fn) {
          var field, flushViaFieldClient, op, path, v;
          path = msg.path, op = msg.op, v = msg.v;
          flushViaFieldClient = function() {
            var fieldClient;
            if (!(fieldClient = field.client(socket.id))) {
              fieldClient = field.registerSocket(socket);
            }
            fieldClient.queue.push([msg, fn]);
            return fieldClient.flush();
          };
          if (!(field = self.otFields[path])) {
            field = self.otFields[path] = new Field(self._adapter, self._pubSub, path, v);
            return self._adapter.get(path, function(err, val, ver) {
              var snapshot;
              snapshot = field.snapshot = (val != null ? val.$ot : void 0) || '';
              return flushViaFieldClient();
            });
          } else {
            return flushViaFieldClient();
          }
        });
        socket.on('txn', function(txn, clientStartId) {
          var base;
          base = transaction.base(txn);
          if (hasInvalidVer(socket, base, clientStartId)) {
            return;
          }
          return commit(txn, function(err, txn) {
            var txnId;
            txnId = transaction.id(txn);
            base = transaction.base(txn);
            if (err && err !== 'duplicate') {
              return socket.emit('txnErr', err, txnId);
            }
            return nextTxnNum(clientId, function(num) {
              return socket.emit('txnOk', txnId, base, num);
            });
          });
        });
        socket.on('txnsSince', txnsSince = function(ver, clientStartId) {
          if (hasInvalidVer(socket, ver, clientStartId)) {
            return;
          }
          return redisClient.get('txnClock.' + clientId, function(err, value) {
            if (err) {
              throw err;
            }
            socket.emit('txnNum', value || 0);
            return forTxnSince(ver, clientId, function(txn) {
              return nextTxnNum(clientId, function(num) {
                socket.__base = transaction.base(txn);
                return socket.emit('txn', txn, num);
              });
            });
          });
        });
        socket.__base = 0;
        clientSockets[clientId] = socket;
        pubSub.subscribe(clientId, paths);
        return txnsSince(ver + 1, clientStartId);
      });
    });
  };
  addSubDatum = function(self, data, root, remainder, value, ver, finish) {
    var appendRoot, nextRoot, nextValue, prop, _ref2;
    if (remainder == null) {
      return data.push([root, value, ver]);
    }
    _ref2 = pathParser.split(remainder), appendRoot = _ref2[0], remainder = _ref2[1];
    for (prop in value) {
      nextRoot = root ? root + '.' + prop : prop;
      nextValue = value[prop];
      if (appendRoot) {
        nextRoot += '.' + appendRoot;
        nextValue = pathParser.fastLookupBreakOnRef(appendRoot, nextValue);
      }
      if (nextValue && nextValue.$r) {
        finish.remainingGets++;
        self.get(nextRoot, function(err, value, ver) {
          addSubDatum(self, data, nextRoot, remainder, value, ver, finish);
          return finish();
        });
      }
      addSubDatum(self, data, nextRoot, remainder, nextValue, ver, finish);
    }
  };
  this._fetchSubData = function(paths, callback) {
    var data, finish, otData, path, remainder, root, _i, _len, _ref2;
    data = [];
    otData = {};
    finish = function() {
      if (!--finish.remainingGets) {
        return callback(null, data, otData);
      }
    };
    finish.remainingGets = paths.length;
    self = this;
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      path = paths[_i];
      _ref2 = pathParser.split(path), root = _ref2[0], remainder = _ref2[1];
      this.get(root, function(err, value, ver) {
        var otField, otPath, otPaths, _j, _len2;
        if (err) {
          return callback(err);
        }
        otPaths = allOtPaths(value, root + '.');
        for (_j = 0, _len2 = otPaths.length; _j < _len2; _j++) {
          otPath = otPaths[_j];
          if (otField = self.otFields[otPath]) {
            otData[otPath] = otField;
          }
        }
        addSubDatum(self, data, root, remainder, value, ver, finish);
        return finish();
      });
    }
  };
  this._forTxnSince = forTxnSince = function(ver, clientId, onTxn, done) {
    if (!pubSub.hasSubscriptions(clientId)) {
      return;
    }
    return redisClient.zrangebyscore('txns', ver, '+inf', 'withscores', function(err, vals) {
      var i, txn, val, _len;
      if (err) {
        throw err;
      }
      txn = null;
      for (i = 0, _len = vals.length; i < _len; i++) {
        val = vals[i];
        if (i % 2) {
          if (!pubSub.subscribedToTxn(clientId, txn)) {
            continue;
          }
          transaction.base(txn, +val);
          onTxn(txn);
        } else {
          txn = JSON.parse(val);
        }
      }
      if (done) {
        return done();
      }
    });
  };
  nextClientId = function(callback) {
    return redisClient.incr('clientClock', function(err, value) {
      if (err) {
        throw err;
      }
      return callback(value.toString(36));
    });
  };
  this.createModel = function() {
    var model;
    model = new Model;
    model.store = self;
    model._ioUri = self._ioUri;
    model.startIdPromise = startIdPromise;
    startIdPromise.on(function(startId) {
      return model._startId = startId;
    });
    nextClientId(function(clientId) {
      return model.clientIdPromise.fulfill(clientId);
    });
    return model;
  };
  this.createStoreModel = function() {
    var key, val, _ref2, _results;
    this.model = this.createModel();
    _ref2 = this.model.async;
    _results = [];
    for (key in _ref2) {
      val = _ref2[key];
      _results.push(this[key] = val);
    }
    return _results;
  };
  this.createStoreModel();
  this.flush = function(callback) {
    var cb, done;
    done = false;
    cb = function(err) {
      if (callback && (done || err)) {
        callback(err);
        callback = null;
      }
      return done = true;
    };
    adapter.flush(cb);
    self = this;
    return redisClient.flushdb(function(err) {
      if (err && callback) {
        callback(err);
        return callback = null;
      }
      redisInfo.onStart(redisClient, cb);
      return self.createStoreModel();
    });
  };
  stm = new Stm(redisClient);
  this._commit = commit = function(txn, callback) {
    var ver;
    ver = transaction.base(txn);
    if (ver && typeof ver !== 'number') {
      throw new Error('Version must be null or a number');
    }
    return stm.commit(txn, function(err, ver) {
      transaction.base(txn, ver);
      if (callback) {
        callback(err, txn);
      }
      if (err) {
        return;
      }
      pubSub.publish(transaction.path(txn), {
        txn: txn
      });
      return txnApplier.add(txn, ver);
    });
  };
  txnApplier = new Serializer({
    withEach: function(txn, ver) {
      var args, method;
      args = transaction.args(txn).slice(0);
      method = transaction.method(txn);
      args.push(ver, function(err) {
        if (err) {
          throw err;
        }
      });
      return adapter[method].apply(adapter, args);
    }
  });
  this.disconnect = function() {
    return [redisClient, subClient, txnSubClient].forEach(function(client) {
      return client.quit();
    });
  };
};
allOtPaths = function(obj, prefix) {
  var k, results, v;
  if (prefix == null) {
    prefix = '';
  }
  results = [];
  if (!(obj && obj.constructor === Object)) {
    return results;
  }
  for (k in obj) {
    v = obj[k];
    if (v && v.constructor === Object) {
      if (v.$ot) {
        results.push(prefix + k);
        continue;
      }
      results.push.apply(results, allOtPaths(v, k + '.'));
    }
  }
  return results;
};

==> racer-0.1.3.tgz: package/lib/Store.js <==
var Field, MemoryAdapter, Model, Promise, PubSub, Serializer, Stm, Store, allOtPaths, pathParser, redis, redisInfo, specHelper, transaction;

redis = require('redis');

MemoryAdapter = require('./adapters/Memory');

Model = require('./Model.server');

Stm = require('./Stm');

PubSub = require('./PubSub');

transaction = require('./transaction');

pathParser = require('./pathParser');

Serializer = require('./Serializer');

specHelper = require('./specHelper');

redisInfo = require('./redisInfo');

Promise = require('./Promise');

Field = require('./mixin.ot/Field.server');

Store = module.exports = function(options) {
  var Adapter, adapter, addSubDatum, clientSockets, commit, db, forTxnSince, hasInvalidVer, host, ignoreSubscribe, localModels, nextClientId, nextTxnNum, onOtMsg, onTxnMsg, otFields, port, pubSub, redisClient, redisOptions, redisStarts, self, startIdPromise, stm, subClient, subscribeToStarts, txnApplier, txnSubClient, _ref;
  if (options == null) options = {};
  self = this;
  Adapter = options.Adapter || MemoryAdapter;
  this._adapter = adapter = new Adapter;
  _ref = redisOptions = options.redis || {}, port = _ref.port, host = _ref.host, db = _ref.db;
  this._redisClient = redisClient = redis.createClient(port, host, redisOptions);
  this._subClient = subClient = redis.createClient(port, host, redisOptions);
  this._txnSubClient = txnSubClient = redis.createClient(port, host, redisOptions);
  this.otFields = otFields = {};
  redisStarts = null;
  startIdPromise = new Promise;
  ignoreSubscribe = false;
  (subscribeToStarts = function(selected) {
    if (ignoreSubscribe) return ignoreSubscribe = false;
    if (db !== void 0 && !selected) {
      return redisClient.select(db, function(err) {
        if (err) throw err;
        return subscribeToStarts(true);
      });
    }
    return redisInfo.subscribeToStarts(subClient, redisClient, function(starts) {
      redisStarts = starts;
      if (startIdPromise.value) startIdPromise.clearValue();
      return startIdPromise.fulfill(starts[0][0]);
    });
  })();
  ignoreSubscribe = true;
  redisClient.on('connect', subscribeToStarts);
  redisClient.on('end', function() {
    redisStarts = null;
    return startIdPromise.clearValue();
  });
  this._localModels = localModels = {};
  onTxnMsg = function(clientId, txn) {
    var base, model, socket;
    if (clientId === transaction.clientId(txn)) return;
    if (model = localModels[clientId]) return model._onTxn(txn);
    if (socket = clientSockets[clientId]) {
      base = transaction.base(txn);
      if (base > socket.__base) {
        socket.__base = base;
        return nextTxnNum(clientId, function(num) {
          return socket.emit('txn', txn, num);
        });
      }
    }
  };
  onOtMsg = function(clientId, ot) {
    var socket;
    if (socket = clientSockets[clientId]) {
      if (socket.id === ot.meta.src) return;
      return socket.emit('otOp', ot);
    }
  };
  this._pubSub = pubSub = new PubSub({
    redis: redisOptions,
    pubClient: redisClient,
    subClient: txnSubClient,
    onMessage: function(clientId, _arg) {
      var ot, txn;
      txn = _arg.txn, ot = _arg.ot;
      if (txn) return onTxnMsg(clientId, txn);
      return onOtMsg(clientId, ot);
    }
  });
  this._nextTxnNum = nextTxnNum = function(clientId, callback) {
    return redisClient.incr('txnClock.' + clientId, function(err, value) {
      if (err) throw err;
      return callback(value);
    });
  };
  hasInvalidVer = function(socket, ver, clientStartId) {
    if (!startIdPromise.value) {
      socket.disconnect();
      return true;
    }
    if (!(clientStartId && clientStartId === startIdPromise.value)) {
      socket.emit('fatalErr');
      return true;
    }
    return false;
  };
  clientSockets = {};
  this._setSockets = function(sockets) {
    return sockets.on('connection', function(socket) {
      return socket.on('sub', function(clientId, paths, ver, clientStartId) {
        var txnsSince;
        if (hasInvalidVer(socket, ver, clientStartId)) return;
        socket.on('disconnect', function() {
          pubSub.unsubscribe(clientId);
          delete clientSockets[clientId];
          return redisClient.del('txnClock.' + clientId, function(err, value) {
            if (err) throw err;
          });
        });
        socket.on('subAdd', function(clientId, paths, callback) {
          pubSub.subscribe(clientId, paths);
          return self._fetchSubData(paths, function(err, data) {
            return callback(data);
          });
        });
        socket.on('subRemove', function(clientId, paths) {
          throw 'Unimplemented: subRemove';
        });
        socket.on('otSnapshot', function(setNull, fn) {
          var TODO, field;
          if (field = self.otFields[path]) return TODO = 'TODO';
        });
        socket.on('otOp', function(msg, fn) {
          var field, flushViaFieldClient, op, path, v;
          path = msg.path, op = msg.op, v = msg.v;
          flushViaFieldClient = function() {
            var fieldClient;
            if (!(fieldClient = field.client(socket.id))) {
              fieldClient = field.registerSocket(socket);
            }
            fieldClient.queue.push([msg, fn]);
            return fieldClient.flush();
          };
          if (!(field = self.otFields[path])) {
            field = self.otFields[path] = new Field(self._adapter, self._pubSub, path, v);
            return self._adapter.get(path, function(err, val, ver) {
              var snapshot;
              snapshot = field.snapshot = (val != null ? val.$ot : void 0) || '';
              return flushViaFieldClient();
            });
          } else {
            return flushViaFieldClient();
          }
        });
        socket.on('txn', function(txn, clientStartId) {
          var base;
          base = transaction.base(txn);
          if (hasInvalidVer(socket, base, clientStartId)) return;
          return commit(txn, function(err, txn) {
            var txnId;
            txnId = transaction.id(txn);
            base = transaction.base(txn);
            if (err && err !== 'duplicate') {
              return socket.emit('txnErr', err, txnId);
            }
            return nextTxnNum(clientId, function(num) {
              return socket.emit('txnOk', txnId, base, num);
            });
          });
        });
        socket.on('txnsSince', txnsSince = function(ver, clientStartId) {
          if (hasInvalidVer(socket, ver, clientStartId)) return;
          return redisClient.get('txnClock.' + clientId, function(err, value) {
            if (err) throw err;
            socket.emit('txnNum', value || 0);
            return forTxnSince(ver, clientId, function(txn) {
              return nextTxnNum(clientId, function(num) {
                socket.__base = transaction.base(txn);
                return socket.emit('txn', txn, num);
              });
            });
          });
        });
        socket.__base = 0;
        clientSockets[clientId] = socket;
        pubSub.subscribe(clientId, paths);
        return txnsSince(ver + 1, clientStartId);
      });
    });
  };
  addSubDatum = function(self, data, root, remainder, value, ver, finish) {
    var appendRoot, nextRoot, nextValue, prop, _ref2;
    if (remainder == null) return data.push([root, value, ver]);
    _ref2 = pathParser.split(remainder), appendRoot = _ref2[0], remainder = _ref2[1];
    for (prop in value) {
      nextRoot = root ? root + '.' + prop : prop;
      nextValue = value[prop];
      if (appendRoot) {
        nextRoot += '.' + appendRoot;
        nextValue = pathParser.fastLookupBreakOnRef(appendRoot, nextValue);
      }
      if (nextValue && nextValue.$r) {
        finish.remainingGets++;
        self.get(nextRoot, function(err, value, ver) {
          addSubDatum(self, data, nextRoot, remainder, value, ver, finish);
          return finish();
        });
      }
      addSubDatum(self, data, nextRoot, remainder, nextValue, ver, finish);
    }
  };
  this._fetchSubData = function(paths, callback) {
    var data, finish, otData, path, remainder, root, _i, _len, _ref2;
    data = [];
    otData = {};
    finish = function() {
      if (!--finish.remainingGets) return callback(null, data, otData);
    };
    finish.remainingGets = paths.length;
    self = this;
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      path = paths[_i];
      _ref2 = pathParser.split(path), root = _ref2[0], remainder = _ref2[1];
      this.get(root, function(err, value, ver) {
        var otField, otPath, otPaths, _j, _len2;
        if (err) return callback(err);
        otPaths = allOtPaths(value, root + '.');
        for (_j = 0, _len2 = otPaths.length; _j < _len2; _j++) {
          otPath = otPaths[_j];
          if (otField = self.otFields[otPath]) otData[otPath] = otField;
        }
        addSubDatum(self, data, root, remainder, value, ver, finish);
        return finish();
      });
    }
  };
  this._forTxnSince = forTxnSince = function(ver, clientId, onTxn, done) {
    if (!pubSub.hasSubscriptions(clientId)) return;
    return redisClient.zrangebyscore('txns', ver, '+inf', 'withscores', function(err, vals) {
      var i, txn, val, _len;
      if (err) throw err;
      txn = null;
      for (i = 0, _len = vals.length; i < _len; i++) {
        val = vals[i];
        if (i % 2) {
          if (!pubSub.subscribedToTxn(clientId, txn)) continue;
          transaction.base(txn, +val);
          onTxn(txn);
        } else {
          txn = JSON.parse(val);
        }
      }
      if (done) return done();
    });
  };
  nextClientId = function(callback) {
    return redisClient.incr('clientClock', function(err, value) {
      if (err) throw err;
      return callback(value.toString(36));
    });
  };
  this.createModel = function() {
    var model;
    model = new Model;
    model.store = self;
    model._ioUri = self._ioUri;
    model.startIdPromise = startIdPromise;
    startIdPromise.on(function(startId) {
      return model._startId = startId;
    });
    nextClientId(function(clientId) {
      return model.clientIdPromise.fulfill(clientId);
    });
    return model;
  };
  this.createStoreModel = function() {
    var key, val, _ref2, _results;
    this.model = this.createModel();
    _ref2 = this.model.async;
    _results = [];
    for (key in _ref2) {
      val = _ref2[key];
      _results.push(this[key] = val);
    }
    return _results;
  };
  this.createStoreModel();
  this.flush = function(callback) {
    var cb, done;
    done = false;
    cb = function(err) {
      if (callback && (done || err)) {
        callback(err);
        callback = null;
      }
      return done = true;
    };
    adapter.flush(cb);
    self = this;
    return redisClient.flushdb(function(err) {
      if (err && callback) {
        callback(err);
        return callback = null;
      }
      redisInfo.onStart(redisClient, cb);
      return self.createStoreModel();
    });
  };
  stm = new Stm(redisClient);
  this._commit = commit = function(txn, callback) {
    var ver;
    ver = transaction.base(txn);
    if (ver && typeof ver !== 'number') {
      throw new Error('Version must be null or a number');
    }
    return stm.commit(txn, function(err, ver) {
      transaction.base(txn, ver);
      if (callback) callback(err, txn);
      if (err) return;
      pubSub.publish(transaction.path(txn), {
        txn: txn
      });
      return txnApplier.add(txn, ver);
    });
  };
  txnApplier = new Serializer({
    withEach: function(txn, ver) {
      var args, method;
      args = transaction.args(txn).slice(0);
      method = transaction.method(txn);
      args.push(ver, function(err) {
        if (err) throw err;
      });
      return adapter[method].apply(adapter, args);
    }
  });
  this.disconnect = function() {
    return [redisClient, subClient, txnSubClient].forEach(function(client) {
      return client.quit();
    });
  };
};

allOtPaths = function(obj, prefix) {
  var k, results, v;
  if (prefix == null) prefix = '';
  results = [];
  if (!(obj && obj.constructor === Object)) return results;
  for (k in obj) {
    v = obj[k];
    if (v && v.constructor === Object) {
      if (v.$ot) {
        results.push(prefix + k);
        continue;
      }
      results.push.apply(results, allOtPaths(v, k + '.'));
    }
  }
  return results;
};
